/**
 * @file
 * A generic XML editor component based on jsTree.
 */

/**
 * To init:
 *   jQuery('#tree_container_selector').xmlTreeEditor('init', '#editor_selector', 'xml_string', config_object);
 *
 *   Where:
 *   - #tree_container_selector: The css selector for the html container that
 *     should contain the tree component.
 *   - #editor_selector: The css selector for the html container that should get
 *     the editor components.
 *   - xml_string: The string of XML to edit.
 *   - config_object: The json object containing the editor configuration.
 *
 *
 * To get the new XML string:
 *   jQuery('#tree_container_selector').xmlTreeEditor('read');
 *
 *   Where:
 *   - #tree_container_selector: The css selector for the html container that
 *     contains the tree component.
 *
 *
 * To search for a specific item in the tree
 *   jQuery('#tree_container_selector').xmlTreeEditor('search', search_string);
 *
 *   Where:
 *   - #tree_container_selector: The css selector for the html container that
 *     contains the tree component.
 *   - search_string: The name of the node. This can also be a path, e.g.
 *     nodeA/nodeB will search nodeBs inside nodeAs.
 *
 *
 *
 * To add a listener:
 *   jQuery('#tree_container_selector').xmlTreeEditor('bind', event, function ());
 *
 *   Where:
 *   - #tree_container_selector: The css selector for the html container that
 *     contains the tree component.
 *   - event string: The type of event to listen for. Currently one of:
 *     - change: Called when anything in the tree changes.
 *   - function: the function be be called when an event of the given type
 *     happens. The arguments of the function depend on the event type.
 *
 */
(function(jQuery) {
  jQuery.fn.xmlTreeEditor = function () {
    /**
     * The data of the current instance of the editor.
     */
    var data = this.data();
    /**
     * The configuration object.
     */
    var config = data.xte_config;
    /**
     * The jQuery selector used to get the div that shows the editor controls.
     */
    var editorSelector = data.xte_editor;
    /**
     * Listeners to global events.
     */
    var listeners = data.listeners;
    /**
     * saveHandlers are specific to the "currently shown attribute editors" and
     * are thus cleared out when a new node is selected for editing.
     */
    var saveHandlers = data.saveHandlers;
    /**
     * A numeric id used to generate unique id's in this tree.
     */
    var nextId = data.nextId;
    /**
     * The (jQuery enhanced) tree.
     */
    var tree = this;

    if (saveHandlers == undefined) {
      saveHandlers = [];
    }

    var type;
    var parent;
    var searchString;
    var attributes;
    var searchStringArray = [];
    var cssSelectorArray = [];
    var returnArray = [];
    var returnObject = {};
    var listener;
    var i;
    switch (arguments[0]) {
      case 'init':
        // Clear the listeners.
        listeners = {
          change:[],
          onloadeditor:[]
        };
        nextId = 1;
        editorSelector = arguments[1];
        var xmlString = jQuery.trim(arguments[2]);
        config = arguments[3];
        initEditor(editorSelector);
        doInit(tree, xmlString, config);
        break;

      case 'read':
        return treeToXml(tree);
        break;

      case 'select':
        createEditorFor(arguments[1]);
        break;

      case 'addNode':
        if (!arguments[1]) {
          addNode();
        }
        else {
          type = arguments[1].toString();
          parent = arguments[2];
          attributes = arguments[3];
          addNode(type, parent, attributes);
        }
        break;

      case 'delNode':
        removeSelectedNode();
        break;

      case 'saveNode':
        updateSelectedNode();
        break;

      case 'search':
        searchString = arguments[1].toString();

        searchStringArray = searchString.split('/');
        for (i = 0; i < searchStringArray.length; i++) {
          cssSelectorArray.push('li[rel=' + searchStringArray[i] + ']');
        }

        returnObject = tree.find(cssSelectorArray.join(' '));
        returnObject.each(function () { //put return object items in 'normal' array
          returnArray.push(jQuery(this));
        });
        return returnArray;
        break;

      case 'bind':
        type = arguments[1].toLowerCase();
        listener = arguments[2];
        if (listeners[type] != undefined) {
          listeners[type].push(listener);
        }
        else {
          alert(Drupal.t("unknown listener type: ") + type);
        }
        break;
    }

    data.xte_config = config;
    data.xte_editor = editorSelector;
    data.listeners = listeners;
    data.saveHandlers = saveHandlers;
    data.nextId = nextId;
    this.data(data);
    return this;

    /**
     * Initialises a tree from data.
     */
    function doInit(treeContainer, xmlString, config) {
      if (xmlString.length == 0) {
        xmlString = "<" + config.valid_children[0] + "/>";
      }
      var treeData = questionStringToTreeObject(xmlString);
      for (var key in config.types) {
        if (config.types[key].icon != undefined) {
          if (config.types[key].icon.image.substring(0, 1) != '/') {
            config.types[key].icon.image = config.basePath + '/assets/' + config.types[key].icon.image;
          }
        }
      }
      var treeConfig = {
        "core": {
          "html_titles": true
        },
        "plugins": [ "themes", "json_data", "ui", "types", "dnd", "crrm", "search" ],
        "json_data": {},
        "ui": {
          "select_limit": "1"
        },
        "themes" : {
          "theme": "classic"
        },
        "search" : {},
        "types" : config
      }
      treeConfig.json_data.data = treeData;
      jQuery(treeContainer).jstree(treeConfig);

      treeContainer.bind("select_node.jstree",
        function(tree) {
          return function (e, args) {
            var element = jQuery(args.args[0].parentNode);
            if (element.length>0) {
              jQuery("#log1").html("Last operation: " + e.type);
              tree.xmlTreeEditor('select', element);
            }
          }
        }(tree)
        );

    }

    /**
     * Reads out a tree and converts it back to XML
     */
    function treeToXml(treeContainer) {
      var container = treeContainer.jstree("core").get_container();
      var root = container[0].children[0].children[0];
      var treeRoot = treeContainer.jstree("core")._get_node(root);

      var xmlDoc;
      if (document.implementation && document.implementation.createDocument) {
        xmlDoc = document.implementation.createDocument("","",null);
      }
      else {
        xmlDoc = new ActiveXObject("MSXML2.DOMDocument");
      }
      if (treeRoot.length > 0) {
        addNodeToXML(treeContainer, xmlDoc, xmlDoc, treeRoot);
      }

      var xmlString = getSerializedXML(xmlDoc);
      return xmlString;
    }

    /**
     * @param treeContainer
     *   The jQuery enhanced DOM object that contains the jsTree.
     * @param xmlDoc
     *   The xml document to add the node to.
     * @param parent
     *   xml element to use as the parent for the new node.
     * @param node
     *   jsTree tree node (jQuery extended li element) to use as basis for the
     *   new node.
     */
    function addNodeToXML(treeContainer, xmlDoc, parent, node) {
      var data = node.data().jstree;
      var children;
      var child;
      var n;
      var childConfig = config.types[data.type];
      if (childConfig.is_group == undefined) {
        var xmlNode = xmlDoc.createElement(data.type);
        for (var attName in data.attributes) {
          var attValue = data.attributes[attName];
          xmlNode.setAttribute(attName, attValue);
        }
        if (data.content.length > 0) {
          InnerHTMLToNode(data.content, xmlNode);
        }
        parent.appendChild(xmlNode);

        children = treeContainer.jstree("core")._get_children(node);
        for ( n = 0; n<children.length; n++) {
          child = treeContainer.jstree("core")._get_node(children[n]);
          addNodeToXML(treeContainer, xmlDoc, xmlNode, child);
        }
      }
      else {
        children = treeContainer.jstree("core")._get_children(node);
        for ( n = 0; n<children.length; n++) {
          child = treeContainer.jstree("core")._get_node(children[n]);
          addNodeToXML(treeContainer, xmlDoc, parent, child);
        }
      }
    }

    /**
     * Creates the DOM elements for the editor.
     */
    function initEditor(editorSelector) {
      var added = jQuery(editorSelector).empty().append('\n\
        <fieldset id="editor_structure">\n\
          <legend>Add/remove:</legend>\n\
          <div class="editor_structure_contents" id="editor_structure_contents">\n\
            <div id="selectedNodeRemoveContainer">\n\
              <a id="selectedNodeRemoveButton" href="javascript:void(0)"><img src="' + config.basePath + '/assets/icons/delete_icon.png" alt="Delete" /> Remove</a> <span>item</span>\n\
            </div>\n\
            <div id="selectedNodeAddContainer">\n\
              <a id="selectedNodeAddButton" href="javascript:void(0)">\n\
              <img src="' + config.basePath + '/assets/icons/add_icon.png" alt="Add" />\n\
              Add</a>\n\
              <select id="selectedNodeAddlist"></select>\n\
              to <span>item</span>\n\
            </div>\n\
          </div>\n\
        </fieldset>\n\
        <fieldset id="editor_values">\n\
          <legend id="editor_values_legend">Edit:</legend>\n\
          <form>\n\
            <div class="editor_values_contents" id="editor_values_contents"></div>\n\
            <div><button type="button" id="selectedNodeUpdateButton">Save</button></div>\n\
          </form>\n\
        </fieldset>\n\
      ');
      added.find('#selectedNodeAddButton').bind('click', function () {
        tree.xmlTreeEditor('addNode');
      });
      added.find('#selectedNodeRemoveButton').bind('click', function () {
        tree.xmlTreeEditor('delNode');
      });
      added.find('#selectedNodeUpdateButton').bind('click', function () {
        tree.xmlTreeEditor('saveNode');
      });
    }

    /**
    * Create an editor for the selected tree item.
    *
    * @param element
    *   The LI tree element to create the editor for.
    */
    function createEditorFor(element) {
      var listenerData;
      var editor, childElement;
      var addButtonWrapper, addButtons;
      var data = element.data().jstree;
      var itemConfig = config.types[data.type];
      var editorElements = {};
      var title = config.types[data.type].title || data.type;
      emptyEditor();

      editor = jQuery("#editor_values_contents");
      addButtonWrapper = jQuery('#selectedNodeAddlist');
      jQuery('#selectedNodeRemoveContainer span').text(title);
      jQuery('#selectedNodeAddContainer span').text(title);

      jQuery("#selectedNodeUpdateButton").text("Save " + title);
      jQuery("#editor_values_legend").html("Edit " + title + ":")

      editorElements.forElement = getEditorElements(element);

      /* add editor for child elements? */
      if (itemConfig.children_in_editor) {
        editorElements.forChildren = [];

        jQuery(itemConfig.children_in_editor).each(function () {
          childElement = element.children("ul").children("[rel='" + this + "']");
          editorElements.forChildren.push(getEditorElements(childElement));
        });
      }
      /* create editor */

      if (editorElements.forElement.attributeEditorElements.length > 0 || editorElements.forChildren != undefined) {
        jQuery("#editor_values").show();
        //for the element
        _appendEditorElements(editor, editorElements.forElement);

        //for its children
        jQuery(editorElements.forChildren).each(function () {
          _appendEditorElements(editor, this);
        });
      }
      else {
        jQuery("#editor_values").hide();
      }

      //add structure buttons for element
      if (editorElements.forElement.structureButtons) {
        if (editorElements.forElement.structureButtons.addNodeOptions && editorElements.forElement.structureButtons.addNodeOptions.length>0) {
          jQuery('#selectedNodeAddContainer').show();
          addButtons = editorElements.forElement.structureButtons.addNodeOptions;
          jQuery(addButtons).each(function () {
            addButtonWrapper.append(this);
          });
        }
        else {
          jQuery('#selectedNodeAddContainer').hide();
        }
      }

      /* call listeners */
      listenerData = {
        "config": config,
        "editor" : jQuery(editorSelector),
        "editorElements": editorElements,
        "treeNode": element,
        "type": data.type
      };


      if (notifyListeners('onLoadEditor', listenerData) === false) {
        /* hide editor */
        jQuery('#editor_values_contents').hide();
      }
      else {
        /* show editor */
        jQuery('#editor_values_contents').show();
      }

      /**
         * If save is pushed, we need to update the title.
         */
      saveHandlers.push(
        function (element) {
          return function () {
            var title = createTitleForElement(element)
            tree.jstree("core").set_text(element, title);
          }
        }(element)
        );

    }

    /**
     * Adds editor elements to the editor
     * @param editor
     *   A DOM node containing the editor
     * @param editorElements
     *   An object containing the editor elements, @see getEditorElements
     **/
    function _appendEditorElements(editor, editorElements) {
      /* add elements to the editor/addButtonWrapper */
      if (editorElements.attributeEditorElements) {
        editorElements.attributeEditorElements.each(function () {
          editor.append(this);
        });
      }

      if (editorElements.description) {
        jQuery(editorElements.description).insertBefore(editor);
      }
    }

    /**
    * Returns editor elements for the selected tree item.
    *
    * @param element
    *   The LI tree element to create the editor for.
    *
    * @return object
    *   An object containing the editor for the attributes, a description, and
    *   some buttons to alter the structure (e.g. add child, remove item) with
    *   the fields:
    *   - attributeEditor: DOM-node
    *   - description: DOM-node
    *   - structureButtons: object
    *     - addNodeOptions: array of DOM-node
    */
    function getEditorElements(element) {
      var attributeEditor;
      var description;
      var attName;
      var data;
      var structureButtons;
      var itemConfig;

      data = element.data().jstree;
      itemConfig = config.types[data.type];

      if (itemConfig != undefined) {
        attributeEditor = jQuery("<div></div>")

        for (attName in itemConfig.attributes) {
          createAttributeEditor(element, itemConfig, data, attName, attributeEditor);
        }

        if (itemConfig.content == 1) {
          createContentEditor(element, data, attributeEditor);
        }
        /* add buttons which allow user to add nodes */
        if (itemConfig.valid_children !== undefined) {
          structureButtons = {};
          structureButtons.addNodeOptions = getAddNodeOptions(itemConfig.valid_children);
        }

        if (itemConfig.description) {
          description = jQuery('<div class="xmlJsonEditor_form_description"><img class="icon" src="' + config.basePath + '/assets/icons/big_info_icon.png" />' + itemConfig.description + '</div>');
        }
      }

      return {
        "attributeEditorElements": attributeEditor.children(),
        "description": description,
        "structureButtons": structureButtons
      };
    }

    /**
     * Creates a list of options for the add-node dropdown.
     *
     * @param valid_children
     *   The array of valid children to put in the dropdown.
     */
    function getAddNodeOptions(valid_children) {
      valid_children = valid_children.sort();
      var numberOfValidChildren = valid_children.length;
      var addNodeOptions = [];

      if (numberOfValidChildren > 0) {
        for (i = 0; i<numberOfValidChildren; i++) {
          var childType = valid_children[i];
          var childTypeConfig = config.types[childType];

          if (!childTypeConfig.hidden || (childTypeConfig.hidden !== true || childTypeConfig.hidden.toString().toLowerCase() !== 'true')) {
            var title = _ucfirst(childTypeConfig.title) || _ucfirst(childType);
            var addChildButton = jQuery('<option value="' + childType + '">' + title + ' </option>');
            addNodeOptions[i] = addChildButton;
          }
        }
      }
      return addNodeOptions
    }

    /**
     * Creates an editor element for an attribute and adds it to the given
     * editor block.
     *
     * @param element
     *   The LI tree element of the item that the attibute belongs to.
     * @param itemConfig
     *   The config data of the item that the attibute belongs to.
     * @param itemData
     *   The data of the item that the attibute belongs to.
     * @param attName
     *   The name of the attribute to create the editor for.
     * @param editor
     *   The editor HTML dom element to add the content editor to.
     */
    function createAttributeEditor(element, itemConfig, itemData, attName, editor) {
      var attributeFormElement, optionElement, attributeWrapperElement, attributeLabelElement, attributeDescriptionElement;
      var hasEditor = true;
      var attConfig = itemConfig.attributes[attName];
      var attValue = "";
      if (itemData.attributes[attName] != undefined) {
        attValue = itemData.attributes[attName];
      }
      var attId = attName;
      var typeString = typeof (attConfig);
      var item ;
      var items;
      var itemValue;
      var itemTitle;
      var valuesArray = [];
      var title;

      /* create wrapper element */
      attributeWrapperElement = jQuery('<div class="xmlJsonEditor_attribute"></div>');

      switch (typeString) {
        /* object provided: create custom form element */
        case "object":
          if (attConfig.alias_of != undefined) {
            hasEditor = false;
            break;
          }
          /* Check if the attribute is depricated. If it is, and if it is empty
           * then don't show it. */
          if (attConfig.depricated == 1) {
            if (attValue.length == 0) {
              break;
            }
            attributeWrapperElement.addClass("depricated");
          }
          title = attName;
          if (attConfig.title != undefined) {
            title = attConfig.title;
          }
          /* hide this form element */
          if (attConfig.hidden && (attConfig.hidden !== true || attConfig.hidden.toString().toLowerCase() !== 'true')) {
            attributeWrapperElement.css("display", "none");
          }

          /* create label element */
          attributeLabelElement = jQuery('<label>' + title + ':</label>');

          /* create form element */
          if (attConfig.values != undefined) {
            attributeFormElement = jQuery('<select id="xmlEditor_' + attId + '" name="' + attId + '" size="1"></select>');

            if (!jQuery.isArray(attConfig.values)) {
              /* config object provided with foreign key/values: get array with values */

              //get foreign node
              items = tree.xmlTreeEditor('search', attConfig.values.node_search_string);

              jQuery(items).each(function () {
                itemData = jQuery(this).data().jstree;
                /* fetch title from foreign node */
                if (attConfig.values.get_titles_from == '/content') {
                  //special occasion: get title from content
                  title = itemData.content;
                }
                else {
                  //get title from attribute
                  title = itemData.attributes[attConfig.values.get_titles_from];
                }

                /* put title and value in object; add object to array */
                valuesArray.push({
                  "title": title,
                  "value": itemData.attributes[attConfig.values.get_values_from]
                });
              });

              /* replace the config values array */
              attConfig.values = valuesArray;
            }

            for (var itemId in attConfig.values) {
              item = attConfig.values[itemId];
              if (typeof(item) == "object") {
                itemValue = item.value;
                itemTitle = item.title;
              }
              else {
                itemValue = item;
                itemTitle = itemValue;
              }

              optionElement = jQuery('<option value="' + itemValue + '">' + itemTitle + '</option>');

              //should this option be selected?
              if (itemValue == attValue.toLowerCase()) {
                optionElement.attr("selected", "selected");
              }

              attributeFormElement.append(optionElement);
            }
          }
          else {
            attributeLabelElement = jQuery('<label>' + title + ':</label>');
            attributeFormElement = jQuery('<input id="xmlEditor_' + attId + '" type="text" value="' + attValue + '" />');
          }

          if (attConfig.description != undefined) {
            attributeDescriptionElement = jQuery('<img class="xmlJsonEditor_help_icon" src="' + config.basePath + '/assets/icons/question_icon.png" title="' + attConfig.description + '" />');
          }

          /* set default value */
          if (attConfig.default_value && !attValue) {
            attributeFormElement.val(attConfig.default_value);
          }

          /* set feedback? */
          if (attConfig.feedback) {
            if (attributeFormElement[0].nodeName.toString().toLowerCase() == 'select') {
              /* give feedback onchange */
              attributeFormElement.change(function () {
                handleAttributeFeedback(jQuery(this), attConfig.feedback);
              });

              //trigger event to show feedback
              attributeFormElement.change();
            }
            else {
              /* give feedback onkeyup */
              attributeFormElement.keyup(function () {
                handleAttributeFeedback(jQuery(this), attConfig.feedback);
              });

              //trigger event to show feedback
              attributeFormElement.keyup();
            }
          }

          /* add styling/css */
          if (attConfig.css) {
            if (jQuery.isArray(attConfig.css)) {
              /* array with class names */
              jQuery(attConfig.css).each(function () {
                attributeFormElement.addClass(this.toString());
              })
            }
            else {
              /* css name/value object */
              attributeFormElement.css(attConfig.css);
            }
          }

          /* append all elements to wrapper */
          attributeWrapperElement.append(attributeLabelElement);
          attributeLabelElement.append(attributeDescriptionElement);
          attributeWrapperElement.append(attributeFormElement);
          break;

        default:
          /* create label and form elements */
          attributeLabelElement = jQuery('<label>' + attName + ':</label>');
          attributeFormElement = jQuery('<input id="xmlEditor_' + attId + '" type="text" value="' + attValue + '" />');

          /* append all elements to wrapper */
          attributeWrapperElement.append(attributeLabelElement);
          attributeWrapperElement.append(attributeFormElement);
          break;
      }

      /* append wrapper element to editor */
      if (attributeWrapperElement.html() != '') {
        /* append feedback element to wrapper */
        attributeWrapperElement.append('<div class="xmlEditorAttributeFeedback"></div>');

        editor.append(attributeWrapperElement);
      }

      /* element has an editor, attach a change listener to it. */
      if (hasEditor) {
        var attEditor = jQuery(".xmlJsonEditor_attribute #xmlEditor_" + attId + "", editor)[0];
        saveHandlers.push(
          function (element, name, attEditor) {
            return function () {
              var data = element.data();
              var newValue = attEditor.value;
              var oldValue = element.data().jstree.attributes[name];
              if (newValue.length > 0 && newValue != " ") {
                data.jstree.attributes[name] = newValue;
              }
              else {
                delete element.data().jstree.attributes[name];
              }
              notifyListeners('change', {
                "element": element,
                "what": "attirbute",
                "which": name
              });
            }
          }(element, attName, attEditor)
          );
      }
    }

    /**
     * Creates an editor element for "content" and adds it to the given editor
     * block.
     *
     * @param element
     *   The LI tree element of the item that the attibute belongs to.
     * @param data
     *   The data of the item to create the editor for.
     * @param editor
     *   The editor HTML dom element to add the content editor to.
     */
    function createContentEditor(element, data, editor) {
      var wrapper  = jQuery("<div class='xmlJsonEditor_attribute'><label>Content:</label></div>");
      var textarea = jQuery("<textarea id='cq_editor_content'></textarea>");

      editor.append(wrapper);
      wrapper.append(textarea);
      wrapper.append('<div class="xmlEditorAttributeFeedback"></div>');

      textarea.keyup(function() {
        var xmlFromString = loadXMLFromString(jQuery(this).val(), true);

        if (xmlFromString.success == false) {
          updateFormElementFeedback(jQuery(this), 'xmlParse', {
            "type": "error",
            "text": 'Warning: This text is not well-formed, which might lead errors. <a href="javascript:void(0)" onclick="jQuery(this).next().toggle()">Technial details</a> <span style="display:none">' + xmlFromString.errorMessage + '</span> / <a href="javascript:void(0)" onclick="jQuery(this).next().toggle()">How to fix</a> <span style="display:none">1) Make sure all tags are closed, e.g. "&lt;p&gt;...&lt;/p&gt;" 2) Fix unsupported HTML entities, e.g. "&amp;amp;rightarr;"</span>'
          })
        }
        else {
          updateFormElementFeedback(jQuery(this), 'xmlParse', undefined);
        }
      });

      var area = jQuery("#cq_editor_content", editor)[0];
      area.value = data.content;
      saveHandlers.push(
        function (element) {
          return function(e) {
            var content = area.value;
            element.data().jstree.content = content;
            notifyListeners('change', {
              "element": element,
              "what": "content"
            });
          }
        }(element)
        );
    }

    /**
     * Creates and set a title for the element.
     *
     * @param element
     *   The LI tree element of the item that needs a title.
     */
    function createTitleForElement(element) {
      var data = element.data().jstree;
      var children = tree.jstree("core")._get_children(element);
      var childData = [];
      for (var i = 0; i < children.length; i++) {
        childData.push(jQuery(children[i]).data().jstree);
      }
      return createTitleForData(data, childData);
    }

    /**
     * Updates the form element feedback
     * @param formElement
     *   DOM-node
     * @param feedbackId
     *   An unique id for this feedback (so it can be removed later on)
     * @param feedbackObject
     *   An object, {"type": string, "text": string} in which "type" is the
     *   feedback type; the function will add a class
     *   xmlEditorAttributeFeedback_<type> to both the form element and the <li>
     *   containing the "text"
     **/
    function updateFormElementFeedback(formElement, feedbackId, feedbackObject) {
      formElement = jQuery(formElement);
      feedbackObject = feedbackObject || {};

      var feedbackMessage = feedbackObject.text;
      var feedbackType    = feedbackObject.type;
      var ul = jQuery('<ul></ul>');

      /* get feedback object from form element */
      var feedbackWrapper = formElement.parent().children('.xmlEditorAttributeFeedback');

      var formElementFeedback = formElement.data('xmlJsonEditor.feedback') || {};

      if (!feedbackMessage) {
        /* remove class from form element */
        if (formElementFeedback[feedbackId] && formElementFeedback[feedbackId].type) {

          formElement.removeClass('xmlEditorAttributeFeedback_' + formElementFeedback[feedbackId].type);
        }

        /* delete feedback object */
        delete formElementFeedback[feedbackId];

      }
      else {
        /* add feedback object */
        if (!formElementFeedback[feedbackId]) {
          formElementFeedback[feedbackId] = {};
        }

        formElementFeedback[feedbackId].text = jQuery('<li class="xmlJsonEditor_feedback_' + feedbackType + '">' + feedbackMessage + '</li>');
        formElementFeedback[feedbackId].type = feedbackType;
      }

      /* put feedback in wrapper */
      for (feedbackId in formElementFeedback) {
        ul.append(formElementFeedback[feedbackId].text);
        formElement.addClass('xmlEditorAttributeFeedback_' + formElementFeedback[feedbackId].type)
      }
      feedbackWrapper.empty();
      feedbackWrapper.append(ul);

      /* store feedback object in form element */
      formElement.data('xmlJsonEditor.feedback', formElementFeedback);
    }

    /**
     * Creates and set a title for the dataset of an element.
     *
     * @param data
     *   The data of an LI tree element of the item that needs a title.
     * @param children
     *   An array of data of the child LI tree elements of the item that needs
     *   a title.
     */
    function createTitleForData(data, children) {
      var type = data.type;
      var typeConfig = config.types[type];
      var content = data.content;
      var title;
      var shortened;
      var oldLength;

      if (typeConfig.title != undefined) {
        title = typeConfig.title;
      }
      else {
        title = _ucfirst(type);
      }

      title = '<em>' + title + '</em>';

      if (typeConfig.atts_in_title != undefined) {
        for (var attId in typeConfig.atts_in_title) {
          var attName = typeConfig.atts_in_title[attId];
          var attValue = data.attributes[attName];
          if (attValue != undefined) {
            title = title + ", " + attName + ": " + attValue;
          }
        }
      }

      if (typeConfig.content != undefined && typeConfig.content) {
        var myRexexp = new RegExp("<[/]?[^<>]*>", "g");

        // Strip out double spacings.
        shortened = content.replace(/(\t|\n|\r)/g," ");
        do {
          oldLength = shortened.length;
          shortened = shortened.replace("  ", " ");
        } while (shortened.length != oldLength);

        // We have an element with content. Show the first bit of content.
        if (shortened.length > 50) {
          title = title + "; " + shortened.replace(myRexexp, " ").substring(0,45) + "...";
        }
        else {
          title = title + "; " + shortened.replace(myRexexp, " ");
        }
      }
      else if (children != undefined && typeConfig.children_in_editor != undefined) {
        for (var i = 0; i < typeConfig.children_in_editor.length; i++) {
          var childType = typeConfig.children_in_editor[i];
          var childConfig = config.types[childType];
          if (childConfig.content != undefined && childConfig.content) {
            for (var j = 0; j < children.length; j++) {
              if (children[j].metadata) {
                var childMeta = children[j].metadata;
              }
              else {
                childMeta = children[j];
              }
              if (childMeta.type == childType) {

                // Strip out double spacings.
                shortened = childMeta.content.replace(/(\t|\n|\r)/g," ");
                do {
                  oldLength = shortened.length;
                  shortened = shortened.replace("  ", " ");
                } while (shortened.length != oldLength);

                myRexexp = new RegExp("<[/]?[^<>]*>", "g");
                // We have an element with content. Show the first bit of content.
                if (shortened.length > 50) {
                  title = title + "; " + shortened.replace(myRexexp, " ").substring(0,45) + "...";
                }
                else {
                  title = title + "; " + shortened.replace(myRexexp, " ");
                }
                break;
              }
            }
          }
        }
      }

      return title;
    }

    /**
     * Clear out the content of the editor divs.
     */
    function emptyEditor() {
      saveHandlers = [];
      var editor = jQuery(editorSelector);
      editor.find(".xmlJsonEditor_form_description").remove();
      editor.find("#editor_values_contents").empty();
      editor.find("#selectedNodeAddlist").children().remove();
      editor.find('#editor_values').children().filter('.xmlJsonEditor_form_description').remove();

      // If the tree is empty, show the root node items in the add dropdown.
      if (treeIsEmpty()) {
        var addNodeOptions = getAddNodeOptions(config.valid_children);
        var addButtonWrapper = jQuery('#selectedNodeAddlist');
        jQuery(addNodeOptions).each(function () {
          addButtonWrapper.append(this);
        });
      }
    }

    /**
     * Checks if the tree is empty.
     *
     * @return true if the tree is empty, false otherwise.
     */
    function treeIsEmpty() {
      var container = tree.jstree("core").get_container();
      var root = container[0].children[0].children[0];
      var treeRoot = tree.jstree("core")._get_node(root);
      return (treeRoot.length == 0)
    }

    /**
     * Converts a closedQuestion XML string to a jsTree tree object.
     */
    function questionStringToTreeObject(xmlString) {
      var xmlDoc1 = parseXml(xmlString);
      var data;

      var count = xmlDoc1.childNodes.length;
      for (var n = 0; n < count; n++) {
        var child = xmlDoc1.childNodes[n];
        if (child.nodeType == 1) {
          data = handleNode(child);
        }
      }
      return data;
    }

    /**
     * Adds the attributes of the DOM element to the target jsTree object.
     */
    function parseAttributes(nodeConfig, node, target) {
      var count = node.attributes.length;

      for (var n = 0; n < count; n++) {
        var attr = node.attributes[n];
        var name = attr.nodeName.toLowerCase();
        var value = attr.nodeValue;
        var attConfig = nodeConfig.attributes[name];
        if (attConfig == undefined) {
          alert(Drupal.t('An unknown parameter "@item" was found in an item of type "@node".\nIf you do not know what to do, contact your technical support for further assitance.', {
            "@item": name,
            "@node": node.nodeName
          }));
        }
        else {
          if (attConfig.alias_of != undefined) {
            name = attConfig.alias_of;
            attConfig = nodeConfig.attributes[name];
          }
          if (attConfig.depricated == 1) {
            alert(Drupal.t('A depricated parameter "@item" was found in an item of type "@node".\nPlease check all items of this type in the tree for information on how to fix this.', {
              "@item": name,
              "@node": node.nodeName
            }));
          }
          if (attConfig.value_aliases != undefined) {
            var aliasOf = attConfig.value_aliases[value.toLowerCase()];
            if (aliasOf != undefined) {
              value = aliasOf;
            }
          }
        }
        target.metadata.attributes[name] = value;
      }
    }

    /**
     * Creates branch for grouping together similar items.
     *
     * @param type
     *   The type of the group.
     * @param title
     *   The title of the group.
     *
     * @return
     *   A json tree-branch.
     */
    function createGroup(type, title) {
      var group = {
        "data": {
          "title": '<em>' + title + '</em>',
          "icon": "folder"
        },
        "state": "open",
        "attr" : {
          "rel": type
        },
        "metadata": {
          "type": type,
          "attributes":{},
          "content":""
        },
        "children": []
      }
      return group;
    }

    /**
     * Adds the chilren of the DOM element to the target jsTree object.
     */
    function parseChildren(nodeConfig, node, target) {
      var groups = {};
      target.children = [];
      var count = node.childNodes.length;
      for (var n = 0; n < count; n++) {
        var child = node.childNodes[n];
        switch (child.nodeType) {
          case 1:
            var data = handleNode(child);
            var childConfig = config.types[data.metadata.type];
            if (childConfig == undefined) {
              alert(Drupal.t("Unknown child: ") + data.metadata.type);
              target.children.push(data);
            }
            else {
              if (childConfig.in_group == undefined) {
                target.children.push(data);
              }
              else {
                var groupName = childConfig.in_group;
                if (groups[groupName] == undefined) {
                  var groupConfig = config.types[groupName];
                  var groupTitle = groupConfig.title == undefined ? groupName : groupConfig.title;
                  groups[groupName] = createGroup(groupName, groupTitle);
                  target.children.push(groups[groupName]);
                }
                var group = groups[groupName];
                group.children.push(data);
              }
            }
            break;
        }
      }
    }

    /**
     * Turn the XML node into a jsTree tree element.
     */
    function handleNode(node) {
      var target = null;
      var tagName = node.tagName.toLowerCase();
      var nodeConfig = config.types[tagName];
      var nodeId = "xmlEditor_" + (nextId++);

      if (nodeConfig == undefined) {
        return {
          "data": {
            "title": "UNKNOWN: " + tagName
          },
          "attr" : {
            "rel": tagName,
            "id": nodeId
          },
          "metadata": {
            "type": tagName,
            "attributes": {},
            "content": ""
          }
        };
      }

      if (nodeConfig.alias_of != undefined) {
        tagName = nodeConfig.alias_of;
        if (nodeConfig.content_to_attribute != undefined) {
          node.setAttribute(nodeConfig.content_to_attribute, getXMLNodeInnerHTML(node));
        }

        nodeConfig = config.types[tagName];
      }

      /* do some configs */
      target = {
        "data": {
          "title": tagName
        },
        "attr" : {
          "rel": tagName,
          "id": nodeId
        },
        "metadata": {
          "type": tagName,
          "attributes": {},
          "content": ""
        }
      };

      //hide node in tree?
      if (nodeConfig.hidden != undefined && nodeConfig.hidden == 1) {
        target.attr.style = "display: none";
      }

      //parse attributes
      parseAttributes(nodeConfig, node, target);

      //do some settings
      if (nodeConfig.max_children == 0) {
        if (nodeConfig.content) {
          var content = getXMLNodeInnerHTML(node);
          target.metadata.content = content;
        }
      }
      else {
        if (nodeConfig.state == undefined) {
          target.state = "open";
        }
        else {
          target.state = nodeConfig.state;
        }
        parseChildren(nodeConfig, node, target);
      }

      // Now we set a pretty title.
      target.data.title = createTitleForData(target.metadata, target.children);

      return target;
    }

    /**
     * Converts XML string to XML DOM
     *
     * @credits http://goessner.net/download/prj/jsonxml/
     */
    function parseXml(xml) {
      var dom = null;
      if (window.DOMParser) {
        try {
          dom = (new DOMParser()).parseFromString(xml, "text/xml");
        }
        catch (e) {
          dom = null;
        }
      }
      else if (window.ActiveXObject) {
        try {
          dom = new ActiveXObject('Microsoft.XMLDOM');
          dom.async = false;
          if (!dom.loadXML(xml)) { // parse error ..
            window.alert(dom.parseError.reason + dom.parseError.srcText);
          }
        }
        catch (e) {
          dom = null;
        }
      }
      else {
        alert(Drupal.t("cannot parse xml string!"));
      }
      return dom;
    }

    /**
     * Returns serialized XML
     *
     * @param xmlObject
     *   A xml object
     *
     * @return string
     */
    function getSerializedXML(xmlObject) {
      var serializer;
      var serialized;
      try {
        // XMLSerializer exists in current Mozilla browsers
        serializer = new XMLSerializer();
        serialized = serializer.serializeToString(xmlObject);
      }
      catch (e) {
        // Internet Explorer has a different approach to serializing XML
        serialized = xmlObject.xml;
      }
      return serialized;
    }

    /**
     * Convert a string to the content of a node.
     */
    function InnerHTMLToNode(xmlString, targetNode) {
      var dom = null;
      var xmlFromString = loadXMLFromString(xmlString, true);

      if (xmlFromString.success == false) {
        dom = targetNode.ownerDocument;
        var textNode = dom.createCDATASection(xmlString);
        targetNode.appendChild(textNode);
      }
      else {
        dom = xmlFromString.dom;
        var count = dom.childNodes[0].childNodes.length;
        for (var n = 0; n < count; n++) {
          var child = dom.childNodes[0].childNodes[n];
          targetNode.appendChild(child.cloneNode(true));
        }
      }
    }

    /**
     * Converts XML string to XML DOM object
     * @param xmlString
     *   The string to convert to XML.
     * @param isInnerXML
     *  Boolean determining whether the xmlString is a full xml string or an
     *  inner xml string.
     *
     * @return object
     *   Object with the fields:
     *   - success: boolean Indication if the conversion was successful.
     *   - dom: XML-DOM object of the given XML.
     *   - errorMessage: string Message if the conversion failed.
     **/
    function loadXMLFromString(xmlString, isInnerXML) {
      var dom = null;
      var returnObject = {};
      var errorMsg;
      if (isInnerXML) {
        xmlString = "<root>" + xmlString + "</root>";
      }

      if (window.DOMParser) {
        dom = (new DOMParser()).parseFromString(xmlString, "text/xml");
        if (dom.documentElement.nodeName == "parsererror" || dom.documentElement.firstChild.nodeName == "parsererror") {
          errorMsg = dom.documentElement.firstChild.textContent;
          dom = null;
        }
      }
      else if (window.ActiveXObject) {
        try {
          dom = new ActiveXObject('Microsoft.XMLDOM');
          dom.async = false;
          if (!dom.loadXML(xmlString)) {
            errorMsg = dom.parseError.reason + dom.parseError.srcText;
            dom = null;
          }
        }
        catch (e) {
          dom = null;
          errorMsg = dom.parseError.reason + dom.parseError.srcText;
        }
      }
      else {
        errorMsg = Drupal.t("cannot parse xml string!");
      }

      if (dom != null) {
        returnObject.success = true;
        returnObject.dom = dom;
      }
      else {
        returnObject.success = false;
        returnObject.errorMessage = errorMsg;
      }

      return returnObject;
    }


    /**
     * Convert the content of the node into a string.
     */
    function getXMLNodeInnerHTML(node) {
      if (node.childNodes.length == 0) {
        // The node has no children to return!
        return "";
      }
      var tagName = node.tagName.toLowerCase();
      var myregexp = new RegExp("<[\/]?(" + tagName + ")[^><]*>", "i");
      var serialized = getSerializedXML(node);
      var shorter = serialized.substr(0, serialized.length-tagName.length-3).replace(myregexp,"");
      return shorter;
    }

    /**
     * Adds a node to the tree
     *
     * @param type
     *   String (optional) determining the type, as defined in the config.
     * @param parent
     *   DOM/Jquery node (optional) The parent to which the new node will be
     *   added.
     * @param attributes
     *   JSON attribute name/values
     * @see http://www.jstree.com/documentation/crrm
     */
    function addNode(type, parent, attributes) {
      type = type || jQuery(editorSelector).find('#selectedNodeAddlist').val();
      parent = parent || null; //currently selected node;
      attributes = attributes || {};
      var nodeId = "xmlEditor_" + (nextId++);
      var position = "last";   //it will be added as last child
      if (treeIsEmpty()) {
        position = 'before';
        parent = -1;
      }
      var newElementConfig = config.types[type];
      var title = _ucfirst(newElementConfig.title) || _ucfirst(type);
      var newNodeConfig = {
        "attr":{
          "rel": type,
          "id": nodeId
        },
        "data": {
          "title": title
        }
      };

      //hide node in tree?
      if (newElementConfig.hidden != undefined && newElementConfig.hidden == 1) {
        newNodeConfig.attr.style = "display: none";
      }

      var callback = function () {
        var i, newElement = arguments[0];
        var auto_children = newElementConfig.auto_children;

        jQuery(newElement).data("jstree", {
          "type": type,
          "attributes": attributes,
          "content": ""
        });

        /* give the element a proper title */
        tree.jstree("rename_node", newElement, createTitleForElement(newElement));

        /* add auto children */
        if (auto_children) {
          for (i = 0; i < auto_children.length; i++) {
            addNode(auto_children[i], newElement);
          }
        }

        /* highlight the new element */
        jQuery(newElement).addClass("xmlJsonEditor_newNodeHighlighted");
        window.setTimeout(function () {
          jQuery(newElement).removeClass("xmlJsonEditor_newNodeHighlighted");
        }, 2000);

      /* select the new element */
      //jQuery(newElement).children("a").click();
      };
      var skip_rename = true;

      tree.jstree("create", parent, position, newNodeConfig, callback, skip_rename);
      notifyListeners('change', {
        "what": "create",
        "element": newNodeConfig
      });

    }

    /**
     * Reads the editor for the selected node and updates the values.
     */
    function updateSelectedNode() {
      for (var i = 0; i < saveHandlers.length; i++) {
        saveHandlers[i].call(null);
      }
      return true;
    }

    /**
     * Removes the currently selected node from the tree.
     */
    function removeSelectedNode() {
      tree.jstree("remove");
      emptyEditor();
      notifyListeners('change', {
        "what": "remove"
      });
    }

    /**
     * Returns the given string, with the first character in upper case.
     *
     * @param str
     *   The string to uppercase the first character of.
     */
    function _ucfirst (str) {
      if (!str) {
        return null;
      }
      var f = str.charAt(0).toUpperCase();
      return f + str.substr(1);
    }

    /**
     * Notify all listeners of type "type" that an event occurred, with the
     * given data.
     *
     * @param type
     *   The type of event.
     * @param data
     *   The data of the event.
     */
    function notifyListeners(type, data) {
      var i, value, returnValue;
      type = type.toLowerCase();
      if (listeners[type]) {
        for (i = 0; i < listeners[type].length; i++) {
          value = listeners[type][i].call(null, data);
          returnValue = returnValue === false ? false : value;
        }
      }

      return returnValue;
    }

    /**
     * Adds feedback to a attribute editor form element
     * @param formElement
     *  A jQueried form DOM element
     * @param feedbackConfigArray
     *  Its feedback config object
     **/
    function handleAttributeFeedback(formElement, feedbackConfigArray) {
      var feedbackCount = feedbackConfigArray.length;
      var i;
      var formElementValue = formElement.val();
      var feedbackObject = {};
      var feedbackConfig;
      var match;

      /* add new feedback */
      for (i=0; i<feedbackCount; i++) {
        feedbackConfig = feedbackConfigArray[i];
        match = new RegExp(feedbackConfig.match);
        if (match.test(formElementValue) == true) {
          /* we have a match, now find out what to do with it */
          feedbackObject.text = feedbackConfig.text;
          if ( feedbackConfig.correct != undefined ) {
            if (feedbackConfig.correct == 1) {
              /* style feedback text */
              feedbackObject.type = 'correct';

              /* enable form submitting */
              jQuery('#selectedNodeUpdateButton').attr('disabled', '');
            }
            else {
              /* style feedback text */
              feedbackObject.type = 'error';

              if (feedbackObject.fatal == 1) {
                /* prevent form from submitting */
                jQuery('#selectedNodeUpdateButton').attr('disabled', 'disabled');
              }
            }

            /* show the feedback text */
            updateFormElementFeedback(formElement, feedbackConfig.match, feedbackObject);
          }

          if (feedbackConfig.stop == 1) {
            break;
          }
        } else {
          /* remove feedback */
          updateFormElementFeedback(formElement, feedbackConfig.match, undefined);
        }
      }
    }
  };
})(jQuery);
