
/**
 * The configuration of the xml editor for ClosedQuestion xml format. This is a
 * json object containing the keys:
 * - valid_children: array of node names that are valid root nodes of the XML.
 * - types: json object containing all allowed nodes as name-config pairs. Each
 *   config object contains the following fields:
 *   - alias_of: (optional) String, the name of another node type that this node
 *     type is an alias of. Before processing the name of this node type is
 *     changed to the name of the real node type. If this is set, all other
 *     configurations are ignored and the configuration of the new name is used,
 *     with the exception of content_to_attribute
 *   - content_to_attribute: Used in combination with alias_of, to indicate that
 *     the content of the old node should be put in a parameter of the new node.
 *   - content: int, 1 if all children of the node are to be treated as a single
 *     string and should thus be displayed in a text-box in the editor. 0 if any
 *     children should be treated as separate nodes.
 *   - attributes: (optional) json object containing name-config pairs for all
 *     valid attributes of the node. The config can be a string or an object. If
 *     the config is a string, nothing special is done. If the config is an
 *     object it can contain the following keys:
 *     fields:
 *     - alias_of: (optional) String, the name of another attribute that this
 *       attribute is an alias of. Before processing the name of this attribute
 *       is changed to the name of the real attribute. If this is set, all other
 *       configurations are ignored and the configuration of the new name is
 *       used.
 *     - title: (optional) The human readable title of the attribute
 *     - descripion: (optional) String with a description of the attribute.
 *     - values: (optional) Either a json array containing valid values of the
 *       attribute or an object specifying the valid values in another way.
 *       If it is an array then each value is either a string or an object. If
 *       the value is a string, that string will be used as the actual value and
 *       as human-readable string for display. If the value is an object it must
 *       contain the following:
 *       - title: The human-readable title of the attribute value.
 *       - value: The actual value for the attribute.
 *       If the values option is an object it must contain the following keys:
 *       - node_search_string: A search string to find the nodes to takes values
 *         from.
 *       - get_values_from: An attribute name that the values should be taken
 *         from.
 *       - get_titles_from: An attribute name that the titles should be taken
 *         from, or "/content" if the title should be taken from the content of
 *         the nodes.
 *     - value_aliases: (optional) Object with oldname/newname combinations.
 *       Used to indicate that a certain (old) value is an alias of another
 *       (new) value. Before processing the old value will be replaced with the
 *       new value.
 *     - depricated: (optional) int 0 or 1. Indicates this attribute is depri-
 *       cated. This will hide the attribute if it has no value, and show a
 *       warning if it does have a value.
 *     - css: (optional) Object with css name/value pairs to style the attribute
 *       editor's form element OR an array with class names to apply to the form
 *       element.
 *     - feedback (optional) Array with objects, allowing for feedback on user
 *       input in the attribute editors. Each object can/should have following
 *       keys:
 *       - correct: int 0 or 1. Indicates that this match refers to a correct
 *         user input.
 *       - match: String containing a regular expression, which will be matched
 *         against the attribute's value
 *       - text: String with the text to show to the user when there is a match.
 *       - fatal: (optional) int 0 or 1. (only available when correct=0)
 *         Indicates whether the error should prevent the user from saving the
 *         node. Default: 0.
 *       - stop: (optional) int 0 or 1. Indicates whether the feedback script
 *         should check for the next feedbacks in the array. Default: 1.
 *   - valid_children: Array of node names that are valid children of this node.
 *   - max_children: Integer indicating the maximum number of children allowed
 *     in this node. -1 for no limit.
 *   - icon: (optional) Object to set a custom tree-icon for this node type.
 *     - image: URL of the image to use as an icon in the tree .
 *   - descripion: (optional) String with a description of the node.
 *   - atts_in_title: (optional) Array of attribute names that should be added
 *     to the title in the tree. Each listed attribute is added to the title as
 *     ", attname: attvalue"
 *   - auto_children: (optional) Array of node names. One node of each of the
 *     given types is automaticly created when a node of the current type is
 *     added to the tree.
 *   - children_in_editor: (optional) Array of node names. Children of the given
 *     types are hidden in the tree and their editor-widgets are added to the
 *     current node.
 *   - hidden: (optional) int 0 or 1. Indicates that this node should be hidden
 *     in the tree. Should probably combined with children_in_editor on the
 *     parent node.
 *   - is_group: (optional) int 0 or 1. Indicates that this node is a virtual
 *     node that is only present in the tree, to group other nodes together, and
 *     not present in the XML.
 *   - in_group: (optional) String name of the group to put this node in. This
 *     group is only present in the tree to group similar nodes together. The
 *     group is not an actual XML node.
 *
 */
var CQ_XML_config = {
  "valid_children": ["question"],
  "types": {
    "question": {
      "content": 0,
      "attributes": {
        "type": {
          "title": "Question Type",
          "values": [" ", "option", "check", "dragdrop", "hotspot", "balance", "value", "fillblanks", "selectorder", "flash", "sequence"],
          "value_aliases": {
            "pattern": "selectorder"
          }
        },
        "inlinefeedback": {
          "title": "Feedback between options",
          "values": [
          {
            "title": "No",
            "value": " "
          },
          {
            "title": "Yes",
            "value": 1
          }
          ],
          "description": "For Multiple-Choice questions only."
        },
        "duplicates": {
          "title": "Allow duplicates",
          "values": [
          {
            "title": "No",
            "value": " "
          },
          {
            "title": "Yes",
            "value": 1
          }
          ],
          "description": "For Select&Order questions only."
        },
        "alignment": {
          "title": "Alignment",
          "description": "For Select&Order questions only.",
          "values": [
          {
            "title": "Default",
            "value": " "
          },
          {
            "title": "Horizontal",
            "value": "horizontal"
          }
          ]
        },
        "optionheight": {
          "title": "Option Height",
          "description": "For Select&Order questions only."
        },
        "width": {
          "depricated": 1,
          "title": "Width (This should be moved to the Matchimg)",
          "feedback" : [
          {
            "correct": 0,
            "match": ".+",
            "text": "Width should be moved to the Matchimg"
          }
          ]
        },
        "height": {
          "depricated": 1,
          "title": "Height (This should be moved to the Matchimg)",
          "feedback" : [
          {
            "correct": 0,
            "match": ".+",
            "text": "Height should be moved to the Matchimg"
          }
          ]
        }
      },
      "valid_children": [
      "text",
      "matheval",
      "hints",
      "correct",
      "options",
      "inlineoption",
      "matchimg",
      "default",
      "startstate",
      "mapping",
      "question"
      ],
      "max_children": -1
    },
    "text": {
      "content": 1,
      "attributes": {},
      "valid_children": [],
      "icon" : {
        "image": "icons/text_icon.png"
      },
      "max_children": 0
    },
    "prompt": {
      "content": 1,
      "attributes": {},
      "valid_children": [],
      "icon" : {
        "image": "icons/text_icon.png"
      },
      "max_children": 0,
      "description": "Shown above the list of options in a MC question, for instance: Pick One"
    },
    "inlinechoice": {
      "content": 0,
      "attributes": {
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        },
        "freeform": {
          "title": "Free-form input",
          "values": [
          {
            "title": "No",
            "value": " "
          },
          {
            "title": "Yes",
            "value": 1
          }
          ]
        },
        "group": "string",
        "class": "string",
        "style": "string"
      },
      "valid_children": [],
      "max_children": 0
    },
    "feedbackblock": {
      "content": 0,
      "attributes": {
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        }
      },
      "valid_children": [],
      "max_children": 0
    },
    "mathresult": {
      "content": 0,
      "attributes": {
        "e": "string",
        "expression": {
          "alias_of": "e"
        }
      },
      "valid_children": [],
      "max_children": 0,
      "icon" : {
        "image": "icons/mathresult_icon.png"
      }
    },
    "matheval":{
      "content": 0,
      "attributes": {
        "e": "string",
        "expression": {
          "alias_of": "e"
        },
        "store": "int"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/matheval_icon.png"
      },
      "atts_in_title": ["store", "e"]
    },
    "hints": {
      "content": 0,
      "attributes": {},
      "valid_children": ["hint"],
      "max_children": -1,
      "is_group": 1
    },
    "hint": {
      "content": 1,
      "attributes": {
        "mintries": "int",
        "maxtries": "int",
        "block": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/hint_icon.png"
      },
      "in_group": "hints",
      "atts_in_title": ["mintries", "maxtries", "block"]
    },
    "correct": {
      "content": 1,
      "attributes": {},
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/correct_icon.png"
      }
    },
    "options": {
      "content": 0,
      "attributes": {},
      "valid_children": ["option"],
      "max_children": -1,
      "is_group": 1
    },
    "option": {
      "content": 0,
      "attributes": {
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        },
        "name": {
          "alias_of": "id"
        },
        "correct": {
          "title": "This option is correct",
          "description": "Describes whether this option is correct",
          "values": [
          {
            "title": "No",
            "value": " "
          },
          {
            "title": "Yes",
            "value": "1"
          },
          {
            "title": "Irrelevant",
            "value": "-1"
          }
          ]
        }
      },
      "atts_in_title": ["id", "correct"],
      "icon": {
        "image": "icons/option_icon.png"
      },
      "valid_children": ["choice", "description", "feedback", "feedbackunselected"],
      "auto_children": ["choice"],
      "children_in_editor": ["choice"],
      "max_children": -1,
      "in_group": "options"
    },
    "choice": {
      "title": "Text",
      "content": 1,
      "attributes": {},
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/text_icon.png"
      },
      "hidden": 1
    },
    "description": {
      "content": 1,
      "attributes": {},
      "valid_children": [],
      "max_children": 0
    },
    "feedback": {
      "content": 1,
      "attributes": {
        "mintries": "int",
        "maxtries": "int",
        "block": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/feedback_icon.png"
      },
      "atts_in_title": ["mintries", "maxtries", "block"]
    },
    "feedbackunselected": {
      "content": 1,
      "attributes": {
        "mintries": "int",
        "maxtries": "int",
        "block": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/feedback_icon.png"
      },
      "atts_in_title": ["mintries", "maxtries", "block"]
    },
    "inlineoption": {
      "content": 1,
      "attributes": {
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        },
        "group": "string"
      },
      "valid_children": [],
      "max_children": 0
    },
    "matchimg": {
      "content": 0,
      "attributes": {
        "src": "string",
        "height": "int",
        "width": "int",
        "maxchoices": "int"
      },
      "valid_children": ["hotspot", "draggable"],
      "max_children": -1
    },
    "hotspot": {
      "content": 1,
      "attributes": {
        "shape": {
          "values": ["rect","circle","poly"]
        },
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        },
        "coords": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/hotspot_icon.png"
      },
      "atts_in_title": ["id"]
    },
    "draggable": {
      "content": 1,
      "attributes": {
        "id": {
          "title": "Identifier",
          "description": "A unique name for this item."
        },
        "identifier": {
          "alias_of": "id"
        },
        "class": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/draggable_icon.png"
      }
    },
    "default": {
      "alias_of": "startstate"
    },
    "startstate": {
      "content": 0,
      "attributes": {
        "value": "string"
      },
      "description": "If the student has not answered the question yet, this 'answer' is used to show the question.",
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/startstate_icon.png"
      }
    },
    "mappings": {
      "content": 0,
      "attributes": {},
      "valid_children": ["mapping"],
      "max_children": -1,
      "is_group": 1
    },
    "mapping": {
      "content": 0,
      "attributes": {
        "correct": {
          "title": "This option is correct",
          "description": "Describes whether this option is correct",
          "values": [
          {
            "title": "No",
            "value": " "
          },
          {
            "title": "Yes",
            "value": "1"
          }
          ]
        },
        "stop": {
          "title": "If this mapping matches",
          "values": [
          {
            "title": "Stop processing",
            "value": "1"
          },
          {
            "title": "Continue processing",
            "value": " "
          }
          ]
        }
      },
      "valid_children": ["and","or","not","match","range","feedback"],
      "max_children": -1,
      "icon": {
        "image": "icons/mapping_icon.png"
      },
      "in_group": "mappings",
      "atts_in_title": ["correct", "stop"]
    },
    "and": {
      "content": 0,
      "attributes": {
      },
      "valid_children": ["and","or","not","match","range"],
      "max_children": -1,
      "icon": {
        "image": "icons/and_icon.png"
      }
    },
    "or": {
      "content": 0,
      "attributes": {
      },
      "valid_children": ["and","or","not","match","range"],
      "max_children": -1,
      "icon": {
        "image": "icons/or_icon.png"
      }
    },
    "not": {
      "content": 0,
      "attributes": {
      },
      "valid_children": ["and","or","not","match","range"],
      "max_children": 1,
      "icon": {
        "image": "icons/not_icon.png"
      }
    },
    "match": {
      "content": 0,
      "attributes": {
        "matchall": {
          "title": "Match all items",
          "values": [
          {
            "title": "No, Stop after first success",
            "value": " "
          },

          {
            "title": "Yes, All must match for success",
            "value": "1"
          }
          ]
        },
        "inlinechoice": "string",
        "inlineoption": "string",
        "pattern": "string",
        "hotspot": "string",
        "hotspotpattern": {
          "alias_of": "hotspot"
        },
        "draggable": "string",
        "draggablepattern": {
          "alias_of": "draggable"
        }
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/match_icon.png"
      },
      "atts_in_title": ["inlinechoice", "inlineoption", "hotspot", "draggable", "pattern"]
    },
    "combination": {
      "alias_of": "match"
    },
    "range": {
      "content": 0,
      "attributes": {
        "matchall": "int",
        "inlinechoice": "string",
        "value": "string",
        "minval": "string",
        "maxval": "string"
      },
      "valid_children": [],
      "max_children": 0,
      "icon": {
        "image": "icons/range_icon.png"
      },
      "atts_in_title": ["inlinechoice", "value", "minval", "maxval"]
    },
    "sequence": {
      "alias_of": "mapping"
    },
    "pattern": {
      "alias_of": "match",
      "content_to_attribute": "pattern"
    }
  }
}
